/****************************************************************************
** Copyright ( C ) 2011-2018 Nicolas Omont and Artelys.  All rights reserved.
** This file is licensed under the Eclipse Public License.
*****************************************************************************/

set AREAS ordered;
set TIMESTAMP ordered;
set TECHNOLOGIES ordered;

param areaConsumption{AREAS, TIMESTAMP} >=0 default 0;

param capacityCost{TECHNOLOGIES} >=0 default 0;
param energyCost{TECHNOLOGIES} >=0 default 0;
param hydroEnergy{AREAS} >=0 default 0;
param maxCapacity{AREAS, TECHNOLOGIES} >=0 default +Infinity;
param maxExchangeCapacity{AREAS,AREAS} >=0 default 0;
param availabilityFactor{AREAS, TIMESTAMP, TECHNOLOGIES} >=0 <=1 default 1;

var capacity{a in AREAS, t in TECHNOLOGIES} >= 0 <= maxCapacity[a,t];
var energy{a in AREAS, h in TIMESTAMP, t in TECHNOLOGIES} >= 0 <= maxCapacity[a,t];
var exchange{a in AREAS, b in AREAS, h in TIMESTAMP : a<>b} >=0 <= maxExchangeCapacity[a,b];

var capacityCosts{a in AREAS, t in TECHNOLOGIES} = capacityCost[t]*card(TIMESTAMP)/8760*capacity[a,t] / 1E6;
var energyCosts{a in AREAS, t in TECHNOLOGIES}   = sum{h in TIMESTAMP} energyCost[t]*energy[a,h,t] / 1E6;
var totalCosts{a in AREAS, t in TECHNOLOGIES}    = capacityCosts[a,t] + energyCosts[a,t];

minimize totalCost : sum{a in AREAS, t in TECHNOLOGIES} totalCosts[a,t];

subject to capacityCtr{a in AREAS, h in TIMESTAMP, t in TECHNOLOGIES } :
	energy[a,h,t] <= capacity[a,t] * availabilityFactor[a,h,t]; 
	
subject to energyCtr{a in AREAS, h in TIMESTAMP } :
	sum{t in TECHNOLOGIES} energy[a,h,t] + sum{ b in AREAS : b <> a} exchange[b,a,h] = areaConsumption[a,h] + sum{ b in AREAS : b <> a} exchange[a,b,h];

subject to storageCtr{a in AREAS, t in TECHNOLOGIES : t = "HydroReservoir"} :
	hydroEnergy[a]*capacity[a,t]/maxCapacity[a,t] >= sum{h in TIMESTAMP} energy[a,h,t]  ;

